#!/usr/bin/env bash

set -e

startdir="$(pwd)"

# yumbootstrap for creating Fedora 8 chroot on host
if [ ! -d "yumbootstrap" ]; then
	printf "==> Preparing yumbootstrap\n"
	mkdir -p yumbootstrap
	curl -L https://github.com/dozzie/yumbootstrap/archive/2733c23d6970d12acf7566d3facc92df507d2a8f.tar.gz | bsdtar -x --strip-components 1 -C yumbootstrap -f -
	cd yumbootstrap
	for patch in ${startdir}/patches/yumbootstrap/*.patch; do
		patch -p1 -i ${patch}
	done
	cd ..
fi

mkdir -p src
cd src

# fake uname kernel release, so we don't bother about obscure issues
if [ ! -d "fakeuname" ]; then
	printf "==> Preparing fakeuname\n"
	mkdir -p fakeuname
	curl -L https://github.com/rpm-software-management/fakeuname/archive/b4a5728a444cb12e7c21487493e659eef4fdc429.tar.gz | bsdtar -x --strip-components 1 -C fakeuname -f -
fi

if [ ! -f Netstream4X_linux.tar.gz ]; then
	printf "==> Downloading sources from Geniatech\n"
	curl -L -O https://opensource.geniatech.eu/Netstream4X/1.1.0.402r1/linux/Netstream4X_linux.tar.gz
fi
if [ ! -d Netstream4X_linux ]; then
	printf "==> Preparing sources from Geniatech\n"
	tar xzf Netstream4X_linux.tar.gz

	tar xzf Netstream4X_linux/synopsis/vixs_firmware/SP5495-2013-09-27/Software/GNU-tools/arc_gcc_rel_4.4_src/arc_gcc_rel4.4.tar.gz
	for patch in ${startdir}/patches/toolchain/*.patch; do
		(cd arc_gcc_rel4.4 && patch -p1 -i "${patch}")
	done
	if [ -e "${startdir}/configs/uclibc.config" ]; then
		cp -f "${startdir}/configs/uclibc.config" arc_gcc_rel4.4/uClibc-0.9.30.3/src/arc_config
	fi

	tar xjf Netstream4X_linux/synopsis/vixs_firmware/SP5495-2013-09-27/Software/linux/arc-linux-2.6.30-src-20130820.tar.bz2
	cp -f Netstream4X_linux/synopsis/vixs_firmware/SP5495-2013-09-27/boot/Makefile arc-linux-2.6.30/arch/arc/boot/.
	cd arc-linux-2.6.30
	patch -p0 -i ../Netstream4X_linux/synopsis/patches/arc-linux-2.6.30-network_kernel_msg.patch
	patch -p1 -i ../Netstream4X_linux/synopsis/patches/arc-linux-2.6.30-perl-fix-defined.patch
	patch -p0 -i ../Netstream4X_linux/synopsis/patches/arc-linux-2.6.30-mtd-partitioning-new-sdk.patch
	patch -p1 -i ../Netstream4X_linux/synopsis/patches/arc-linux-2.6.30-sendmmsg.patch
	cd ..
	if [ -e "${startdir}/configs/linux.config" ]; then
		cp -f "${startdir}/configs/linux.config" arc-linux-2.6.30/.config
	else
		cp -f Netstream4X_linux/synopsis/configs/linux-2.6.30-new-sdk_defconfig arc-linux-2.6.30/.config
	fi

	tar xzf Netstream4X_linux/synopsis/vixs_firmware/zlib-1.2.8.tgz
	rm -f zlib-1.2.8/.configured
	make -C zlib-1.2.8 distclean

	tar xzf Netstream4X_linux/synopsis/vixs_firmware/busybox-1.21.1.tgz
	make -C busybox-1.21.1 distclean
	if [ -e "${startdir}/configs/busybox.config" ]; then
		cp -f "${startdir}/configs/busybox.config" busybox-1.21.1/.config
	else
		cp -f Netstream4X_linux/synopsis/configs/busybox_defconfig busybox-1.21.1/.config
	fi

	tar xzf Netstream4X_linux/synopsis/vixs_firmware/libiconv-1.14.tgz
	rm -f libiconv-1.14/.configured
	make -C libiconv-1.14 distclean
	for patch in ${startdir}/patches/libiconv/*.patch; do
		(cd libiconv-1.14 && patch -p1 -i "${patch}")
	done

	cp -f Netstream4X_linux/TombeaVersion.h .
	cp -f Netstream4X_linux/synopsis/source_vixs/rootfs.lst .
	patch -p1 -i "${startdir}/patches/initramfs/adjust-rootfs-list.patch"
fi

cd "${startdir}"
printf "==> Preparing initramfs skeleton\n"
# create all basic dirs
mkdir -p $(awk '/^dir.*/ {printf " initramfs" $2}' src/rootfs.lst)
# copy ready files
awk '/^file.*rfs-arc.*/ {system("cp -f src/Netstream4X_linux/synopsis/vixs_firmware/arc-linux-2.6.30/" $3 " initramfs" $2)}' src/rootfs.lst
awk '/^file.*SYNOPSIS.*/ {sub(/\${SYNOPSIS_ROOT}/, "src/Netstream4X_linux/synopsis", $3); system("cp -f " $3 " initramfs" $2)}' src/rootfs.lst
cp -f src/Netstream4X_linux/synopsis/firmware/Tombea initramfs/usr/bin/.
# fix bits on some files
nonexe=('etc' 'lib/firmware' 'lib/modules/2.6.30' 'lib/transcoding')
for d in ${nonexe[@]}; do find initramfs/${d}/. -maxdepth 1 -type f -exec chmod 644 "{}" +; done
exe=('etc/init.d')
for d in ${exe[@]}; do find initramfs/${d}/. -maxdepth 1 -type f -exec chmod 755 "{}" +; done
cd initramfs && patch -p1 -i "${startdir}/patches/initramfs/adjust-files-to-1.1.0-402r1-firmware.patch"

cd "${startdir}"
# prepare extra sources
extras=(extras/*/)
for pkg in ${extras[@]}; do
	( . "${pkg}$(basename ${pkg}).sh"
	cd src
	if [ ! -d "${SRCDIR}" ]; then
		mkdir -p "${SRCDIR}"
		printf "%s\n" "==> Preparing $(basename ${pkg})"
		curl -L "${SRCURL}" | bsdtar -x --strip-components 1 -C "${SRCDIR}" -f -
		cd "${SRCDIR}"
		# hot-patch the configure script (https://lists.freedesktop.org/archives/pkg-config/2010-March/000508.html)
		[ -f "configure" ] && sed -e 's,atleast-pkgconfig-version ,atleast-pkgconfig-version=,' -i configure
		for patch in ${startdir}/${pkg}patches/*.patch; do
			[ -f "${patch}" ] && patch -p1 -i "${patch}"
		done || true
	fi )
done
