#!/usr/bin/env bash

set -e

startdir=$(pwd)
builduseruid=$(id -u)
buildusergid=$(id -g)

toolchain="/opt/arc_gcc_rel4.4"
target="arc-linux-uclibc"
fakelnxrel="$(ls /lib/modules)"
fakeuname="env LD_PRELOAD=/usr/lib/fakeuname.so FAKE_KERNEL_RELEASE=${fakelnxrel}"
jobs="-j$(grep -c "processor" /proc/cpuinfo | tr -d '\n')"

extras=(openssl dropbear libmnl iptables libdvbcsa minisatip lua xupnpd libusb oscam octoscan pshare ntfs-3g)

fakeuname() {
	cd ${startdir}/src/fakeuname/src
	[ ! -f ".built" ] && make && touch .built
	if [ ! -f ".installed" ]; then
		sudo cp -f fakeuname.so /usr/lib/.
		sudo ldconfig
		touch .installed
	fi
	cd ${startdir}
}

toolchain() {
	cd ${startdir}/src/arc_gcc_rel4.4
	if [ ! -f ".installed" ]; then
		# own /opt for now, so we don't have to patch toolchain script
		sudo chown -R ${builduseruid}:${buildusergid} /opt
		$fakeuname ./build-all.sh --linux-dir ../arc-linux-2.6.30 --install-dir "${toolchain}"
		ln -f -s "$(basename ${toolchain})" /opt/arc
		ln -f -s arc /opt/arc-elf
		ln -f -s arc /opt/arc-uclibc
		# build host utils
		PATH="${toolchain}/bin:${PATH}" make -C uClibc-0.9.30.3/src hostutils
		cp -f uClibc-0.9.30.3/src/utils/ldconfig.host "${toolchain}/bin/${target}-ldconfig"
		cp -f uClibc-0.9.30.3/src/utils/ldd.host "${toolchain}/bin/${target}-ldd"
		# finalize
		sudo chown -R root:root /opt
		printf "${toolchain}/i686-pc-linux-gnu/${target}/lib\n" | sudo tee /etc/ld.so.conf.d/arc.conf
		sudo ldconfig
	fi && touch .installed
	cd ${startdir}
}

basics() {
	cd ${startdir}/src
	cd zlib-1.2.8
	[ ! -f ".configured" ] \
	&& $fakeuname CHOST="${target}" ARFLAGS="cru" ./configure --prefix=/ \
	&& touch .configured

	[ ! -f ".built" ] \
	&& $fakeuname make ${jobs} \
	&& touch .built

	[ ! -f ".installed" ] \
	&& $fakeuname sudo make PATH="${PATH}" DESTDIR="${toolchain}/${target}" install \
	&& touch .installed
	cd ..

	cd busybox-1.21.1
	[ ! -f ".built" ] \
	&& $fakeuname make ${jobs} \
	&& touch .built

	[ ! -f ".installed" ] \
	&& $fakeuname sudo make \
		PATH="${PATH}" \
		CONFIG_PREFIX="${startdir}/initramfs" \
		install \
	&& touch .installed
	cd ..

	cd libiconv-1.14
	[ ! -f ".configured" ] \
	&& $fakeuname ./configure --host=${target} --prefix=/ --disable-static \
	&& touch .configured

	[ ! -f ".built" ] \
	&& $fakeuname make ${jobs} \
	&& touch .built

	[ ! -f ".installed" ] \
	&& $fakeuname sudo make PATH="${PATH}" DESTDIR="${toolchain}/${target}" install \
	&& touch .installed
	cd ..

	cd ${startdir}

	printf "==> Basic initramfs assembly\n"
	# copy basic libs
	find ${toolchain}/${target}/lib/. -maxdepth 1 -iname '*.so*' -exec cp -f -P -t initramfs/lib/. "{}" +
	# create nodes
	awk '/^nod.*/ {system("sudo mknod -m " $3 " initramfs" $2 " " $6 " " $7 " " $8)}' src/rootfs.lst
	# create symbolic links
	awk '/^slink.*/ {system("ln -s -f " $3 " initramfs" $2)}' src/rootfs.lst
}

extras() {
	cd ${startdir}/src
	for pkg in ${extras[@]}; do
		(
			set -e
			. "${startdir}/extras/${pkg}/${pkg}.sh"
			cd "${SRCDIR}"

			[ -n "${CONFIGURE}" ] \
			&& [ ! -f ".configured" ] \
			&& printf "==> Building ${pkg}\n" \
			&& $fakeuname bash -c "${CONFIGURE}" \
			&& touch .configured

			[ ! -f ".built" ] \
			&& $fakeuname bash -c "${BUILD}" \
			&& touch .built

			mkdir -p destdir
			$fakeuname make PATH="${PATH}" ${DESTDIRVARNAME:-DESTDIR}="${PWD}/destdir" install || true

			[ -n "${INSTALL}" ] \
			&& [ ! -f ".installed" ] \
			&& $fakeuname bash -c "${INSTALL}" \
			&& touch .installed

			[ -n "${FILES}" ] \
			&& for e in "${FILES[@]}"; do
				local a=(${e//|/ })
				local td="$(dirname ${startdir}/initramfs/${a[1]})"
				[ ! -d "${td}" ] && mkdir -p "${td}"
				cp -f -R "${a[0]}" "${startdir}/initramfs/${a[1]}" || true
			done

#			[ -d "${startdir}/extras/${pkg}/files" ] \
#			&& cp -f -R "${startdir}/extras/${pkg}/files/." "${startdir}/initramfs/."
		) 2>&1 | tee ${pkg}.log
	done
	# make sure no unneeded files are added to target
	rm -f -R "${startdir}/initramfs/lib/pkgconfig"
	rm -f "${startdir}"/initramfs/lib/*.a "${startdir}"/initramfs/lib/*.la
}

initramfs() {
	[ -d "${startdir}/files" ] && cp -f -R "${startdir}/files/." "${startdir}/initramfs/."
	printf "==> Striping symbols from binaries\n"
	cd ${startdir}/initramfs
	find bin sbin usr/bin usr/sbin -maxdepth 1 -type f -exec ${target}-strip --strip-all "{}" + || true
	# never strip the xcode5drv.ko module
	find lib -type f ! -name xcode5drv.ko -exec ${target}-strip --strip-all "{}" + || true
	printf "==> Building the Linux with initramfs\n"
	cd ${startdir}/src/arc-linux-2.6.30
	# hot-patch our initramfs tree location
	sed -e "s,CONFIG_INITRAMFS_SOURCE=.*,CONFIG_INITRAMFS_SOURCE=\"${startdir}/initramfs\"," -i .config
	# build each target one at a time, so we don't trip when building with multiple jobs
	$fakeuname make ${jobs} objconvert
	$fakeuname make ${jobs} modules
	find . -name '*.ko' -exec ${target}-strip --strip-all "{}" + -exec cp -f -t ${startdir}/initramfs/lib/modules/2.6.30/. "{}" +
	$fakeuname make ${jobs} vmlinux
	gzip -f vmlinux.mem
	./mkimage \
		-A arc \
		-O linux \
		-C gzip \
		-T kernel \
		-a 0x18001000 \
		-e 0x18001000 \
		-n "Arc Linux Kernel" \
		-d vmlinux.mem.gz ${startdir}/uImage_arc_gz_xc5
	cd ${startdir}
}

# main
fakeuname
toolchain || exit 1
export PATH="${toolchain}/bin:${PATH}"
export PKG_CONFIG="pkg-config --define-variable=prefix=${toolchain}/${target} --define-variable=exec_prefix=${toolchain}/${target} $@"
export PKG_CONFIG_LIBDIR="${toolchain}/${target}/lib/pkgconfig"
basics || exit 1
extras
initramfs || exit 1
